--[[

$Revision: 24 $

(C) Copyright 2009 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Initialization

local L = LibStub("AceLocale-3.0"):GetLocale("BossNotes")


----------------------------------------------------------------------
-- Constants

BOSS_NOTES_PREFIX = "BossNotes"

local KEY_PATTERNS = {
	"^encounter:%d+$",
	"^instance:[%a%d]+$",
	"^general$"
}

local CHECKS = {
	request = {
		key = {
			type = "string",
			func = function (key)
				for _, pattern in ipairs(KEY_PATTERNS) do
					if string.match(key, pattern) then return true end
				end
				return false
			end
		}
	},
	response = {
		key = {
			type = "string",
			func = function (key)
				for _, pattern in ipairs(KEY_PATTERNS) do
					if string.match(key, pattern) then return true end
				end
				return false
			end
		},
		class = {
			type = "string",
			func = function (class) return RAID_CLASS_COLORS[class] end
		},
		note = {
			type = "string",
			func = function (note) return #note <= 1000 end
		},
		time = {
			type = "number",
			func = function (time) return time >= 0 end
		},
		clientVersion = {
			type = "string",
			func = function (clientVersion) return string.match(clientVersion, "^%d+%.%d+%.%d+$") end
		}
	}
}


---------------------------------------------------------------------
-- Subsystem

-- Initializes the data subsystem
function BossNotes:InitializeComm ()
end

-- Enables the data subsystem
function BossNotes:EnableComm ()
	self:RegisterComm(BOSS_NOTES_PREFIX)
end

-- Disables the data subsystem
function BossNotes:DisableComm  ()
end


----------------------------------------------------------------------
-- Event handlers

-- Handles comm messages
function BossNotes:OnCommReceived (prefix, message, distribution, sender)
	-- Check distribution
	if not (distribution == "GUILD" and self.db.global.syndicationGuild or
		distribution == "RAID" and self.db.global.syndicationRaid) then
		return
	end
	
	-- Deserialize and validate
	local success, object = self:Deserialize(message)
	if not success or not self:CheckObject(object) then
		return
	end
	
	-- Process
	if object.type == "request" then
		self:SendMessage("BOSS_NOTES_REQUEST", object.key, distribution)
	elseif object.type == "response" then
		self:SendMessage("BOSS_NOTES_RESPONSE", object.key, sender,
				object.class, object.note, object.time,
				object.clientVersion)
	end
end


----------------------------------------------------------------------
-- APIs

-- Sends a request
function BossNotes:SendRequest (key)
	-- Create message
	local request = {
		type = "request",
		key = key
	}
	local message = self:Serialize(request)
	
	-- Send
	if self.db.global.syndicationGuild and IsInGuild() then
		self:SendCommMessage(BOSS_NOTES_PREFIX, message, "GUILD")
	end
	if self.db.global.syndicationRaid and UnitInRaid("player") then
		self:SendCommMessage(BOSS_NOTES_PREFIX, message, "RAID")
	end
end

-- Sends a response
function BossNotes:SendResponse (key, class, note, time, clientVersion, distribution)
	-- Create message
	local response = {
		type = "response",
		key = key,
		class = class,
		note = note,
		time = time,
		clientVersion = clientVersion
	}
	local message = self:Serialize(response)
	
	-- Send
	self:SendCommMessage(BOSS_NOTES_PREFIX, message, distribution)
end


----------------------------------------------------------------------
-- Support methods

-- Checks an incoming object
function BossNotes:CheckObject (object)
	-- Is it a table?
	if type(object) ~= "table" then
		return false
	end
	
	-- Known type?
	local checks = CHECKS[object.type]
	if not checks then
		return false
	end

	-- Run checks
	for name, check in pairs(checks) do
		if type(object[name]) ~= check.type then
			return false
		end
		if not check.func(object[name]) then
			return false
		end
	end
	return true
end
